use std::{
    collections::HashSet,
    io::Read,
    net::{IpAddr, Ipv4Addr},
};

use crate::{
    error::AppResult,
    serde_addr::{TargetAddr, LOCALHOST_V4},
};

#[derive(serde::Deserialize)]
pub struct Config {
    #[serde(default = "default_host")]
    pub host: IpAddr,
    #[serde(default = "default_port")]
    pub port: u16,
    #[serde(default)]
    pub local: HashSet<TargetAddr>,
    #[serde(default)]
    pub remote: HashSet<TargetAddr>,
}

pub fn parse() -> AppResult<Config> {
    if let Some(mut file) = std::env::var_os("CONFIG")
        .and_then(|path| std::fs::File::open(path).ok())
        .or_else(|| std::fs::File::open("config.toml").ok())
    {
        let mut buf = String::new();
        file.read_to_string(&mut buf)?;
        Ok(toml::from_str(&buf)?)
    } else {
        let mut local = HashSet::new();
        local.insert((LOCALHOST_V4, 5232).into());

        let remote = HashSet::new();

        Ok(Config {
            host: default_host(),
            port: default_port(),
            local,
            remote,
        })
    }
}

#[inline]
const fn default_host() -> IpAddr {
    IpAddr::V4(Ipv4Addr::new(0, 0, 0, 0))
}

#[inline]
const fn default_port() -> u16 {
    5233
}
