use socks5_server::proto::Error as SocksProtoError;
use tokio::net::TcpStream;

use crate::serde_addr::TargetAddr;

pub type AppResult<T> = core::result::Result<T, AppError>;

#[derive(Debug)]
pub enum AppError {
    IoError(std::io::Error),
    InvalidConfig(toml::de::Error),
    SocksError(SocksProtoError),
    DataNotUtf8(std::string::FromUtf8Error),
    ThirdPartyHost(TargetAddr),
}

impl From<std::io::Error> for AppError {
    #[inline]
    fn from(value: std::io::Error) -> Self {
        Self::IoError(value)
    }
}

impl From<toml::de::Error> for AppError {
    #[inline]
    fn from(value: toml::de::Error) -> Self {
        Self::InvalidConfig(value)
    }
}

impl From<SocksProtoError> for AppError {
    #[inline]
    fn from(value: SocksProtoError) -> Self {
        Self::SocksError(value)
    }
}

impl From<std::string::FromUtf8Error> for AppError {
    #[inline]
    fn from(value: std::string::FromUtf8Error) -> Self {
        Self::DataNotUtf8(value)
    }
}

pub type HandlerResult<T> = core::result::Result<T, HandlerError>;

#[derive(Debug)]
pub struct HandlerError {
    pub inner: AppError,
    pub stream: TcpStream,
}

impl From<(AppError, TcpStream)> for HandlerError {
    #[inline]
    fn from(value: (AppError, TcpStream)) -> Self {
        Self {
            inner: value.0,
            stream: value.1,
        }
    }
}

impl From<(SocksProtoError, TcpStream)> for HandlerError {
    #[inline]
    fn from(value: (SocksProtoError, TcpStream)) -> Self {
        Self {
            inner: AppError::SocksError(value.0),
            stream: value.1,
        }
    }
}

impl From<(std::io::Error, TcpStream)> for HandlerError {
    #[inline]
    fn from(value: (std::io::Error, TcpStream)) -> Self {
        Self {
            inner: AppError::IoError(value.0),
            stream: value.1,
        }
    }
}
