use std::{
    net::{SocketAddr, ToSocketAddrs},
    sync::Arc,
};

use crate::{
    config::Config,
    error::{AppError, AppResult},
    serde_addr::{AddrIter, TargetAddr, LOCALHOST_V4},
};

pub fn resolve_domain(host: Vec<u8>, port: u16, config: Arc<Config>) -> AppResult<AddrIter> {
    let host = String::from_utf8(host)?;
    let target = TargetAddr::DomainName(host, port);

    if config.local.contains(&target) {
        Ok(SocketAddr::new(LOCALHOST_V4, port).into())
    } else if let Some(remote) = config.remote.get(&target) {
        Ok(remote.to_socket_addrs()?)
    } else {
        Err(AppError::ThirdPartyHost(target))
    }
}

pub fn resolve_ip(addr: SocketAddr, config: Arc<Config>) -> AppResult<SocketAddr> {
    let target = TargetAddr::IpAddress(addr);

    if config.local.contains(&target) {
        Ok(SocketAddr::new(LOCALHOST_V4, addr.port()))
    } else if let Some(remote) = config.remote.get(&target) {
        Ok(remote.to_socket_addrs()?.next().unwrap())
    } else {
        Err(AppError::ThirdPartyHost(target))
    }
}
